<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet version="2.0" 
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
    xmlns:fo="http://www.w3.org/1999/XSL/Format"
    xmlns:axf="http://www.antennahouse.com/names/XSL/Extensions">
    
    <!-- xslt / antennahouse -->
    <xsl:param name="sort_utils">icu.sort</xsl:param>
    <xsl:param name="language">en</xsl:param>
    <xsl:param name="genstring-index-header-numbers-and-symbols">Numbers and Symbols</xsl:param>
    <xsl:param name="toc-nr-separator">P</xsl:param>
    <xsl:param name="id-separator">,&#xA0;</xsl:param>

    <xsl:key name="id-letter-key" match="xe" use="translate(substring(normalize-space(primary),1,1),$lowercase,$uppercase)"/>

    <xsl:key name="id-level1-key" match="xe" use="concat(translate(substring(normalize-space(primary),1,1),$lowercase,$uppercase),' ',normalize-space(primary))"/>
    
    <xsl:key name="id-level2-key" match="xe" use="concat(translate(substring(normalize-space(primary),1,1),$lowercase,$uppercase),' ',normalize-space(primary), ' ' , normalize-space(secondary))"/>

    <xsl:key name="xeLevel1-and-2" match="xe" use="concat(normalize-space(primary),' ',normalize-space(secondary))"/>    
    
    <xsl:variable name="lowercase" select="'áéíóőúűćłóśźżńąęабвгдеёжзийклмнопрстуфхцчшщъыьэюяabcdefghijklmnopqrstuvwxyzäöüÄÖÜ1234567890*+-/\@#|,;.:_~?=)(%$§!²³{[]}€µ&lt;&gt;&amp;'"/>
    <xsl:variable name="uppercase" select="'ÁÉÍÓŐÚŰĆŁÓŚŹŻNAEАБВГДЕЁЖЗИЙКЛМНОПPСТУФХЦЧШЩЪЫЬЭЮЯABCDEFGHIJKLMNOPQRSTUVWXYZÄÖÜÄÖÜ###########################################'"/>

    <!-- INDEX -->
    <xsl:template match="index">
        <xsl:param name="root_pos"/>
        <fo:page-sequence master-reference="mcFlow">
            <fo:flow flow-name="xsl-region-body">
                <fo:block>
                    <xsl:variable name="temp_title">
                        <index>
                            <title>Index</title>
                        </index>
                    </xsl:variable>
                    <!-- choose how to build the index -->
                    <xsl:choose>
                        <!-- debug -->
                        <xsl:when test="$sort_utils = 'show_xml'">
                            <xsl:call-template name="show_index_xml"/>
                        </xsl:when>
                        <!-- std -->
                        <xsl:otherwise>
                            <xsl:call-template name="process-index"/>
                        </xsl:otherwise>
                    </xsl:choose>
                </fo:block>
                <!--<fo:block>
                    <xsl:attribute name="id">EOF0123456EOF789<xsl:value-of select="$root_pos"/></xsl:attribute>
                </fo:block>-->
            </fo:flow>
        </fo:page-sequence>
    </xsl:template>
    
    <xsl:template name="show_xml">
        <xsl:param name="xml_source"/>
        <!-- DUMMY -->
    </xsl:template>
    
    <xsl:template name="show_index_xml">
        <xsl:call-template name="show_xml">
            <xsl:with-param name="xml_source" select="//xe"/>                            
        </xsl:call-template>
        
        <xsl:variable name="groups" select="//xe[generate-id(.)=generate-id(key('id-letter-key',translate(substring(normalize-space(primary),1,1),$lowercase,$uppercase)))]"/>
        
        <fo:block background-color="yellow" page-break-before="always">
            <fo:block color="red" font-size="15pt" font-weight="bold" margin="6pt 6pt 6pt 6pt">
                Groupes
            </fo:block>
            <xsl:call-template name="show_xml">
                <xsl:with-param name="xml_source" select="$groups"/>
            </xsl:call-template>
        </fo:block>
        
        <fo:block background-color="rgb(255,235,235)" page-break-before="always">
            <fo:block color="red" font-size="15pt" font-weight="bold" margin="6pt 6pt 6pt 6pt">
                Sorted
            </fo:block>
            <xsl:variable name="sorted">
            <sorted>
                <xsl:for-each select="$groups">
                    <xsl:sort select="normalize-space(primary)" data-type="text" case-order="upper-first" order="ascending" lang="{$language}"/>
                        <xsl:choose>
                            <xsl:when test="translate(substring(normalize-space(primary),1,1),$lowercase,$uppercase) = '#'">
                                <xsl:value-of select="$genstring-index-header-numbers-and-symbols"/>
                            </xsl:when>
                            <xsl:otherwise>
                                <xsl:value-of select="translate(substring(normalize-space(primary),1,1),$lowercase,$uppercase)"/>
                            </xsl:otherwise>
                        </xsl:choose>
                    <xsl:variable name="id1" select="key('id-letter-key',translate(substring(normalize-space(primary),1,1),$lowercase,$uppercase))"/>
                    <xsl:for-each select="$id1[generate-id(.)=generate-id(key('id-level1-key',concat(translate(substring(normalize-space(primary),1,1),$lowercase,$uppercase), ' ' ,normalize-space(primary))))]">
                        <xsl:sort select="normalize-space(primary)" lang="{$language}"/>
                        <xsl:variable name="first-lonely-primary-id">
                            <xsl:for-each select="key('id-level1-key',concat(translate(substring(normalize-space(primary),1,1),$lowercase,$uppercase), ' ' ,normalize-space(primary)))">
                                <xsl:if test="not(child::secondary)">
                                    <xsl:value-of select="generate-id()"/>
                                </xsl:if>
                            </xsl:for-each>
                        </xsl:variable>
                            <xsl:copy-of select="primary"/>
                            <xsl:for-each select="key('id-level1-key',concat(translate(substring(normalize-space(primary),1,1),$lowercase,$uppercase), ' ' ,normalize-space(primary)))">
                                <xsl:sort select="secondary" lang="{$language}"/>
                            </xsl:for-each>
                            <xsl:variable name="id2" select="key('id-level1-key',concat(translate(substring(normalize-space(primary),1,1),$lowercase,$uppercase), ' ' ,normalize-space(primary)))"/>
                            <xsl:for-each select="$id2[generate-id(.)=generate-id(key('id-level2-key',concat(translate(substring(normalize-space(primary),1,1),$lowercase,$uppercase), ' ' , normalize-space(primary), ' ' ,normalize-space(secondary))))]">
                                <xsl:sort select="normalize-space(secondary)" lang="{$language}"/>
                                <xsl:if test="secondary">
                                        <xsl:copy-of select="secondary"/>
                                </xsl:if>
                            </xsl:for-each>
                    </xsl:for-each>
                </xsl:for-each>
            </sorted>
            </xsl:variable>
            <xsl:call-template name="show_xml">
                <xsl:with-param name="xml_source" select="$sorted"/>
            </xsl:call-template>
        </fo:block>
    </xsl:template>

    <xsl:template name="process-index">
        <!-- LEVEL 1: GROUPING/SORTING by LETTER -->
        <xsl:variable name="groups" select="//xe[generate-id(.)=generate-id(key('id-letter-key',translate(substring(normalize-space(primary),1,1),$lowercase,$uppercase)))]"/>
        <xsl:for-each select="$groups">            
            <xsl:sort select="normalize-space(primary)" data-type="text" case-order="upper-first" order="ascending" lang="{$language}"/>
            <xsl:comment><xsl:copy-of select="."/></xsl:comment>
            <fo:block>                
                <xsl:choose>                    
                    <xsl:when test="translate(substring(normalize-space(primary),1,1),$lowercase,$uppercase) = '#'">
                        <xsl:value-of select="$genstring-index-header-numbers-and-symbols"/>
                    </xsl:when>                
                    <xsl:otherwise>
                        <xsl:value-of select="translate(substring(normalize-space(primary),1,1),$lowercase,$uppercase)"/>
                    </xsl:otherwise>
                </xsl:choose>                
            </fo:block>
            <!-- LEVEL 2: GROUPING/SORTING by element <primary> -->
            <xsl:variable name="id1" select="key('id-letter-key',translate(substring(normalize-space(primary),1,1),$lowercase,$uppercase))"/>
            <xsl:for-each select="$id1[generate-id(.)=generate-id(key('id-level1-key',concat(translate(substring(normalize-space(primary),1,1),$lowercase,$uppercase), ' ' ,normalize-space(primary))))]">
                <xsl:sort select="normalize-space(primary)" lang="{$language}"/>
                <xsl:variable name="first-lonely-primary-id">
                    <xsl:for-each select="key('id-level1-key',concat(translate(substring(normalize-space(primary),1,1),$lowercase,$uppercase), ' ' ,normalize-space(primary)))">
                        <xsl:if test="not(child::secondary)">
                            <xsl:value-of select="generate-id()"/>
                        </xsl:if>
                    </xsl:for-each>
                </xsl:variable>
                <fo:block>
                    <!-- REQCOS-73 -->
                    <xsl:attribute name="keep-with-next.within-column">
                        <!--<xsl:message>idx-prim-pos: <xsl:value-of select="position()"/> - [<xsl:value-of select=".//primary"/>]</xsl:message>-->
                        <xsl:choose>
                            <xsl:when test="position()=1 and not(secondary) and not(position()=last())">1000</xsl:when>
                            <xsl:otherwise>auto</xsl:otherwise>
                        </xsl:choose>
                    </xsl:attribute>
                    <xsl:if test="$first-lonely-primary-id != ''">
                        <xsl:attribute name="text-align-last">justify</xsl:attribute>
                    </xsl:if>
                    <xsl:apply-templates select="primary" mode="idx"/>
                    <xsl:if test="$first-lonely-primary-id != ''">
                        <xsl:value-of select="$toc-nr-separator"/>
                        <fo:leader keep-with-next="always"/>
                        <xsl:value-of select="$toc-nr-separator"/>
                    </xsl:if>
                    <xsl:for-each select="key('id-level1-key',concat(translate(substring(normalize-space(primary),1,1),$lowercase,$uppercase), ' ' ,normalize-space(primary)))">
                        <xsl:sort select="secondary" lang="{$language}"/>
                        <xsl:choose>
                            <xsl:when test="not(child::secondary) and position()!=1">
                                <xsl:value-of select="$id-separator"/>
                            </xsl:when>
                        </xsl:choose>
                        <xsl:if test="not(child::secondary)">
                            <fo:basic-link internal-destination="{generate-id()}">
                                Yadda
                            </fo:basic-link>
                        </xsl:if>
                    </xsl:for-each>
                    <!-- LEVEL 3: GROUPING/SORTING by element <secondary> -->
                    <xsl:variable name="id2" select="key('id-level1-key',concat(translate(substring(normalize-space(primary),1,1),$lowercase,$uppercase), ' ' ,normalize-space(primary)))"/>
                    <xsl:for-each select="$id2[generate-id(.)=generate-id(key('id-level2-key',concat(translate(substring(normalize-space(primary),1,1),$lowercase,$uppercase), ' ' , normalize-space(primary), ' ' ,normalize-space(secondary))))]">
                        <xsl:sort select="normalize-space(secondary)" lang="{$language}"/>
                        <xsl:if test="secondary">
                            <fo:block>
                                <!-- REQCOS-73 -->
                                <xsl:attribute name="keep-with-previous.within-column">
                                    <!--<xsl:message>idx-sec-pos: <xsl:value-of select="position()"/> - [<xsl:value-of select=".//secondary"/>]</xsl:message>-->
                                    <xsl:choose>
                                        <xsl:when test="position()=1 or position()=2 or position()=last()">1000</xsl:when>
                                        <xsl:otherwise>auto</xsl:otherwise>
                                    </xsl:choose>
                                </xsl:attribute>
                                <xsl:apply-templates select="secondary" mode="idx"/>
                                <xsl:value-of select="$toc-nr-separator"/>
                                <fo:leader keep-with-next="always"/>
                                <xsl:value-of select="$toc-nr-separator"/>
                                <xsl:for-each select="key('id-level2-key',concat(translate(substring(normalize-space(primary),1,1),$lowercase,$uppercase), ' ' ,normalize-space(primary), ' ' , normalize-space(secondary)))">
                                    <xsl:if test="position()!=1"><xsl:value-of select="$id-separator"/></xsl:if>
                                    <fo:basic-link internal-destination="{generate-id()}">
                                        Yadda
                                    </fo:basic-link>
                                </xsl:for-each>
                            </fo:block>
                        </xsl:if>
                    </xsl:for-each>
                </fo:block>
            </xsl:for-each>
        </xsl:for-each>
    </xsl:template>
    
    <xsl:template match="xe">
        <xsl:comment> - = [match:xe] = - </xsl:comment>
        <fo:block-container id="{generate-id()}"/>  
    </xsl:template>
    
    <xsl:template match="xe[ancestor::title]">
        <xsl:comment> - = [match:xe] = - </xsl:comment>
        <fo:inline-container id="{generate-id()}"/>  
    </xsl:template>
    
    <xsl:template match="primary | secondary" mode="idx">
        <xsl:apply-templates/>
    </xsl:template>
    
    <!-- workarround REQCOS-95 -->
    <xsl:template match="primary/text()">
        <xsl:choose>
            <xsl:when test="starts-with(.,'//-')">
                <xsl:value-of select=" substring-after(.,'//')"/>                    
            </xsl:when>
            <xsl:otherwise><xsl:value-of select="."/></xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    
</xsl:stylesheet>
